using FitConnect;
using FitConnect.Models;
using FluentAssertions;
using Microsoft.Extensions.Logging;

namespace E2ETest;

public class StraightForwardTest : EndToEndTestBase {
    private string _caseId = null!;
    private string _submissionId = null!;

    [Order(10)]
    [Test]
    public void SendingSubmission() {
        var outgoing = SubmissionBuilder.WithDestination(Settings.DestinationId)
            .WithServiceType("Straight forward test", Settings.LeikaKey)
            .WithAttachments(new Attachment("Test.pdf", "A simple PDF"))
            .WithJsonData(@"{""data"":""value""}")
            .Build();

        var submission = Sender.SendAsync(outgoing).Result;

        _caseId = submission.CaseId!;
        _submissionId = submission.Id!;

        _caseId.Should().NotBeNull();
        _submissionId.Should().NotBeNull();
    }

    [Test]
    [Order(20)]
    public void Sender_GetSubmissionState() {
        // Act
        var status = Sender.GetStatusForSubmissionAsync(_caseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(30)]
    public void Subscriber_GetSubmissionState() {
        // Act
        var status = Subscriber.GetStatusForSubmissionAsync(_caseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }

    [Test]
    [Order(40)]
    public void RequestSubmission() {
        var subscriberWithSubmission = Subscriber.RequestSubmissionAsync(_submissionId).Result;

        var data = subscriberWithSubmission.GetDataJson();
        Logger.LogInformation("Data {Data}", data);

        data.Should().Be(@"{""data"":""value""}");

        var attachments = subscriberWithSubmission.GetAttachmentsAsync().Result;
        attachments.First().Filename.Should().Be("Test.pdf");

        subscriberWithSubmission.AcceptSubmissionAsync().Wait();
    }

    [Test]
    [Order(50)]
    public void Sender_GetSubmissionState_AfterAccepting() {
        // Act
        var status = Sender.GetStatusForSubmissionAsync(_caseId, Settings.DestinationId).Result;

        // Assert
        status.Count.Should().BeGreaterThan(0);
        status.ForEach(
            s => Logger.LogInformation("Status {When} {Event}", s.EventTime, s.EventType));
    }
}
